module Qrcx
  
  class Error < StandardError; end
  class InvalidOrUnsupportedURL < Error; end
  class Timeout < Error; end
  class RedirectURLUnsupported < InvalidOrUnsupportedURL; end

  #ruby standard library
  require 'net/http'
  require 'cgi'
  require 'uri'

  #throws QRCX::Error, QRCX::InvalidOrUnsupportedURL (< Error), QRCX::Timeout (< Error)
  #       QRCX::RedirectURLUnsupported (< InvalidOrUnsupportedURL)
  #throws all Errors that Net::HTTP.get_response may throw
  def self.shorten(url)
    url = CGI.escape(url)
    retry_count = 0
    begin
      short_url = Net::HTTP.get_response(URI.parse("http://qr.cx/api/?longurl=#{url}")).body
    rescue Timeout::Error
      retry_count = retry_count + 1
      retry if retry_count <= 1
      raise Timeout.new($!)
   end

    #error handling
    if short_url.include? "error: either unsupported url or the url is not valid"
      response = Net::HTTP.get_response(URI.parse(CGI.unescape(url)))
      raise RedirectURLUnsupported.new("#{response.code} #{response.message} (#{CGI.unescape(url)})") if response.code =~ /3\d\d/
      raise InvalidOrUnsupportedURL 
    end
    
    raise Error.new(short_url) if short_url.include? "error"

    return short_url
  end

  def self.makelong(url)
    url = CGI.escape(url)
    retry_count = 0

    begin
      short_url = Net::HTTP.get_response(URI.parse("http://qr.cx/api/?get=#{url}")).body
    rescue Timeout::Error
      retry_count = retry_count + 1
      retry if retry_count <= 1
      raise Timeout
    end
    raise Error.new(short_url) if short_url.include? "error"
    
    return short_url
  end
end
